window.Gleam.initSearch([{"doc":"prng","title":"prng","content":"# prng\n\n[![Package Version](https://img.shields.io/hexpm/v/prng)](https://hex.pm/packages/prng)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/prng/)\n\n🎲 A pure random number generator (prng) for Gleam\n\n&gt; ⚙️ This package works for both the Erlang and JavaScript target\n\n## Installation\n\nTo add this package to your Gleam project:\n\n```sh\ngleam add prng\n```\n\n## Generating random values\n\nThis package can help you generate any kind of random values you can think of.\nIt can be useful in many different scenarios: when you need to simulate non\ndeterministic actions, like rolling a dice or flipping a coin; when you need to\nwrite [property based tests](https://ferd.ca/property-based-testing-basics.html);\nor to make fun interactive games where you can spawn randomly generated enemies!\n\n### A mindset shift\n\nThe way random values are generated may be a bit confusing at first, especially\ncoming from other languages like JavaScript. In such languages random number\ngeneration can be as simple as this:\n\n```ts\nconst random_value: number = Math.random()\n```\n\nHowever, this should raise a lot of questions: what if I need those numbers to\nbe in a certain range?\nHow can I generate more complex values, like lists of\nnumbers?\nCan I set the random seed to get deterministic and _reproducible results_ in a\ntest environment?\n\nThe `random` package tries to address all these questions by providing a nice\ninterface to define random value _generators_.\nLet&#39;s have a look at an example to get a taste of what generating random values\nwill look like:\n\n```gleam\nlet generator: Generator(Float) = random.float(0.0, 1.0)\nlet random_value: Float = random.sample_once(generator)\n```\n\nNotice a subtle but fundamental difference: you&#39;re no longer simply generating\na value, you&#39;re _describing_ the values you want to generate and the you can\ntake those out of a generator with a variety of functions, like `sample_once`.\n\nThis neat trick can give two great features:\n\n- _Composability:_ it&#39;s easy to describe simple generators and compose them\n  together to generate complex data structures in an expressive way. The library\n  has a rich API to create and compose generators, you can have a look at it\n  [here](https://hexdocs.pm/prng/).\n- _Reproducibility:_ you can decide the random seed used to generate the random\n  values from a generator. The algorithm for the pseudo number generation will\n  always yield the same results given the same starting seed!  \n  The `random` package also goes out of its way to make sure that the random\n  number generation _works exactly the same on all Gleam targets,_ so you won&#39;t\n  get any discrepancies just by compiling to different targets\n\n## References\n\nThis package and its documentation is based on the awesome\n[Elm implementation](https://package.elm-lang.org/packages/elm/random/1.0.0/)\nof [Permuted Congruential Generators](https://www.pcg-random.org).\nThey have a great documentation full of clear and useful examples; if you are\ncurious, give it a look and show Elm some love!\n\n## Contributing\n\nIf you think there&#39;s any way to improve this package, or if you spot a bug don&#39;t\nbe afraid to open PRs, issues or requests of any kind! Any contribution is\nwelcome 💜\n","url":"index.html"},{"doc":"prng/seed","title":"Seed","content":"pub type Seed\n\n\nSynonyms:\nprng/seed.Seed\nprng/seed Seed","url":"prng/seed.html#Seed"},{"doc":"prng/seed","title":"new","content":"pub fn new(int: Int) -&gt; Seed\n\nSynonyms:\nprng/seed.new\nprng/seed new","url":"prng/seed.html#new"},{"doc":"prng/seed","title":"to_int","content":"pub fn to_int(seed: Seed) -&gt; Int\n\nSynonyms:\nprng/seed.to_int\nprng/seed to_int","url":"prng/seed.html#to_int"},{"doc":"prng/seed","title":"prng/seed","content":"","url":"prng/seed.html"},{"doc":"prng/random","title":"Generator","content":"pub opaque type Generator(a)\n A `Generator(a)` is a data structure that _describes_ how to produce random\n values of type `a`.\n \n Take for example the following generator for random integers:\n \n \n let dice_roll: Generator(Int) = random.int(1, 6)\n \n \n It is just describing the values that can be generated - in this case the\n numbers from 1 to 6 - but it is not actually producing any value.\n\n ## Getting values out of a generator\n \n To actually get a value out of a generator you can use the `step` function:\n it takes a generator and a `Seed` as input and produces a new seed and a\n random value of the type described by the generator:\n \n \n import prng/random\n import prng/seed\n\n let #(roll_result, updated_seed) = dice_roll |&gt; random.step(seed.new(11))\n\n roll_result\n // -&gt; 3\n \n \n The generator is completely deterministic: this means that - given the same\n seed - it will always produce the same results, no matter how many times you\n call the `step` function.\n \n `step` will produce an updated seed that you can use for subsequent calls to\n get different pseudo-random results:\n \n \n let initial_seed = seed.new(11)\n let #(first_roll, new_seed) = dice_roll |&gt; random.step(initial_seed)\n let #(second_roll, _) = dice_roll |&gt; random.step(new_seed)\n \n #(first_roll, second_roll)\n // -&gt; #(3, 2)\n \n \n\nSynonyms:\nprng/random.Generator\nprng/random Generator","url":"prng/random.html#Generator"},{"doc":"prng/random","title":"max_int","content":"pub const max_int: Int = 2_147_483_647\n The underlying algorith will work best for integers in the inclusive range \n going from `min_int` up to `max_int`.\n \n It can generate values outside of that range, but they are &quot;not as random&quot;.\n \nSynonyms:\nprng/random.max_int\nprng/random max_int","url":"prng/random.html#max_int"},{"doc":"prng/random","title":"min_int","content":"pub const min_int: Int = -2_147_483_648\n The underlying algorith will work best for integers in the inclusive range \n going from `min_int` up to `max_int`.\n \n It can generate values outside of that range, but they are &quot;not as random&quot;.\n \nSynonyms:\nprng/random.min_int\nprng/random min_int","url":"prng/random.html#min_int"},{"doc":"prng/random","title":"choose","content":"pub fn choose(one: a, or other: a) -&gt; Generator(a)\n Generates two values with equal probability.\n \n This is a shorthand for `random.uniform(one, [other])`, but can read better\n when there&#39;s only two choices.\n \n ## Examples\n \n Given the following type to model the outcome of a coin flip:\n \n \n pub type CoinFlip {\n   Heads\n   Tails\n }\n \n \n You can write a generator for coin flip outcomes like this:\n \n \n let flip = random.choose(Heads, Tails)\n \n \nSynonyms:\nprng/random.choose\nprng/random choose","url":"prng/random.html#choose"},{"doc":"prng/random","title":"constant","content":"pub fn constant(value: a) -&gt; Generator(a)\n Always generates the given value, no matter the seed used.\n \n ## Examples\n \n \n let always_eleven = random.constant(11)\n random.sample_once(always_eleven)\n // -&gt; 11\n \n \nSynonyms:\nprng/random.constant\nprng/random constant","url":"prng/random.html#constant"},{"doc":"prng/random","title":"float","content":"pub fn float(from: Float, to: Float) -&gt; Generator(Float)\n Generates floating point numbers in the given inclusive range.\n \n ## Examples\n \n \n let probability = random.float(0.0, 1.0)\n  \n \nSynonyms:\nprng/random.float\nprng/random float","url":"prng/random.html#float"},{"doc":"prng/random","title":"int","content":"pub fn int(from: Int, to: Int) -&gt; Generator(Int)\n Generates integers in the given inclusive range.\n \n ## Examples\n \n Say you want to model the outcome of a dice, you could use `int` like this:\n \n \n let dice_roll = random.int(1, 6)\n \n \nSynonyms:\nprng/random.int\nprng/random int","url":"prng/random.html#int"},{"doc":"prng/random","title":"list","content":"pub fn list(from generator: Generator(a), of length: Int) -&gt; Generator(\n  List(a),\n)\n Generates a lists of a fixed size; its values are generated using the\n given generator.\n \n ## Examples\n \n Imagine you&#39;re modelling a game of\n [Risk](https://en.wikipedia.org/wiki/Risk_(game)); when a player &quot;attacks&quot;\n they can roll three dice. You may model that outcome using `list` like this:\n \n \n let dice_roll = random.int(1, 6)\n let attack_outcome = random.list(dice_roll, 3)\n  \n \nSynonyms:\nprng/random.list\nprng/random list","url":"prng/random.html#list"},{"doc":"prng/random","title":"map","content":"pub fn map(generator: Generator(a), with fun: fn(a) -&gt; b) -&gt; Generator(\n  b,\n)\n Transforms the values produced by a generator using the given function.\n \n ## Examples\n \n Imagine you want to make a generator for boolean values that returns\n `True` and `False` with the same probability. You could do that using `map`\n like this:\n \n \n let bool_generator = random.int(1, 2) |&gt; random.map(fn(n) { n == 1 })\n \n \n Here `map` allows you to transform the values produced by the initial\n integer generator - either 1 or 2 - into boolean values: when the original\n generator produces a 1, `bool_generator` will produce `True`; when the\n original generator produces a 2, `bool_generator` will produce `False`.\n \nSynonyms:\nprng/random.map\nprng/random map","url":"prng/random.html#map"},{"doc":"prng/random","title":"map2","content":"pub fn map2(one: Generator(a), other: Generator(b), with fun: fn(\n    a,\n    b,\n  ) -&gt; c) -&gt; Generator(c)\n Combines two generators into a single one. The resulting generator produces\n values obtained by applying `fun` to the values generated by the given\n generators.\n \n ## Examples\n \n Imagine you need to generate random points in a 2D space:\n \n \n pub type Point {\n   Point(x: Float, y: Float)\n }\n \n \n You can compose two basic generators into a `Point` generator using `map2`:\n \n \n let x_generator = random.float(-1.0, 1.0)\n let y_generator = random.float(-1.0, 1.0)\n let point_generator = map2(x_generator, y_generator, Point)\n \n \n &gt; Notice how you could get the same result using `then`:\n &gt; \n &gt; \n &gt; pub fn point_generator() -&gt; Generator(Point) {\n &gt;   use x &lt;- random.then(random.float(-1.0, 1.0))\n &gt;   use y &lt;- random.then(random.float(-1.0, 1.0))\n &gt;   random.constant(Point(x, y))\n &gt; }\n &gt; \n &gt;\n &gt; the `use` syntax paired with `then` may be confusing for other people\n &gt; reading your code, especially Gleam newcomers.\n &gt;\n &gt; Usually `map2`/`map3`/... will be more than enough if you just need to\n &gt; combine simple generators into more complex ones.\n \nSynonyms:\nprng/random.map2\nprng/random map2","url":"prng/random.html#map2"},{"doc":"prng/random","title":"map3","content":"pub fn map3(one: Generator(a), two: Generator(b), three: Generator(\n    c,\n  ), with fun: fn(a, b, c) -&gt; d) -&gt; Generator(d)\n Combines three generators into a single one. The resulting generator\n produces values obtained by applying `fun` to the values generated by the\n given generators.\n \n ## Examples\n \n Imagine you&#39;re writing a generator for random enemies in a game you&#39;re\n making:\n \n \n pub type Enemy {\n   Enemy(health: Int, attack: Int, defense: Int)\n }\n \n \n Each enemy starts with a random health (that can go from 50 to 100) and\n random values for the `attack` and `defense` stats (each can be in a range\n from 1 to 5):\n \n \n let health_generator = random.int(50, 100)\n let attack_generator = random.int(1, 5)\n let defense_generator = random.int(1, 5)\n \n let enemy_generator =\n   random.map3(\n     health_generator,\n     attack_generator,\n     defense_generator,\n     Enemy,\n   )\n \n \nSynonyms:\nprng/random.map3\nprng/random map3","url":"prng/random.html#map3"},{"doc":"prng/random","title":"map4","content":"pub fn map4(one: Generator(a), two: Generator(b), three: Generator(\n    c,\n  ), four: Generator(d), with fun: fn(a, b, c, d) -&gt; e) -&gt; Generator(\n  e,\n)\n Combines four generators into a single one. The resulting generator\n produces values obtained by applying `fun` to the values generated by the\n given generators.\n \nSynonyms:\nprng/random.map4\nprng/random map4","url":"prng/random.html#map4"},{"doc":"prng/random","title":"map5","content":"pub fn map5(one: Generator(a), two: Generator(b), three: Generator(\n    c,\n  ), four: Generator(d), five: Generator(e), with fun: fn(\n    a,\n    b,\n    c,\n    d,\n    e,\n  ) -&gt; f) -&gt; Generator(f)\n Combines five generators into a single one. The resulting generator\n produces values obtained by applying `fun` to the values generated by the\n given generators.\n \n &gt; There&#39;s no `map6`, `map7`, and so on. If you feel like you need to compose\n &gt; together even more generators, you can use the `random.then` function.\n \nSynonyms:\nprng/random.map5\nprng/random map5","url":"prng/random.html#map5"},{"doc":"prng/random","title":"pair","content":"pub fn pair(one: Generator(a), with other: Generator(b)) -&gt; Generator(\n  #(a, b),\n)\n Generates pairs of values obtained by combining the values produced by the\n given generators.\n \n ## Examples\n \n \n let one_to_five = \n \n \nSynonyms:\nprng/random.pair\nprng/random pair","url":"prng/random.html#pair"},{"doc":"prng/random","title":"sample","content":"pub fn sample(from generator: Generator(a)) -&gt; Iterator(a)\n Turns the given generator into an infinite stream of random values generated\n with it.\n \n The initial seed is chosen randomly so you won&#39;t have control over which\n values are generated and may get different results each time you call this\n function.\n \n If you want to have control over the initial seed used to get the infinite\n sequence of values, you can use `to_iterator`.\n \nSynonyms:\nprng/random.sample\nprng/random sample","url":"prng/random.html#sample"},{"doc":"prng/random","title":"sample_once","content":"pub fn sample_once(generator: Generator(a)) -&gt; a\n Generates a single value using the given generator.\n \n The initial seed is chosen randomly so you won&#39;t have control over which\n value is generated and may get different results each time you call this\n function.\n \n This is useful if you want to quickly get a value out of a generator and\n do not care about reproducibility (if you want to decide which seed is\n used for the generation process you&#39;ll have to use `random.step`).\n \n ## Examples\n \n Imagine you want to perform some action, say only 40% of the times.\n Your code may look like this:\n \n \n let probability = random.float(0.0, 1.0)\n case random.sample_once(probability) &lt;= 0.4 {\n   True -&gt; perform_action()\n   False -&gt; Nil // do nothing\n }\n \n \nSynonyms:\nprng/random.sample_once\nprng/random sample_once","url":"prng/random.html#sample_once"},{"doc":"prng/random","title":"step","content":"pub fn step(generator: Generator(a), seed: Seed) -&gt; #(a, Seed)\n Steps a `Generator(a)` producing a random value of type `a` using the given\n seed as the source of randomness.\n \n The stepping logic is completely deterministic. This means that, given a\n seed and a generator, you&#39;ll always get the same result.\n \n This is why this function also returns a new seed that can be used to make\n subsequent calls to `step` to get other random values.\n \n ## Examples\n \n \n let initial_seed = seed.new(11)\n let dice_roll = random.int(1, 6)\n let #(first_roll, new_seed) = random.step(dice_roll, initial_seed)\n let #(second_roll, _) = random.step(dice_roll, new_seed)\n\n #(first_roll, second_roll)\n // -&gt; #(3, 2)\n \n \nSynonyms:\nprng/random.step\nprng/random step","url":"prng/random.html#step"},{"doc":"prng/random","title":"then","content":"pub fn then(generator: Generator(a), do generator_from: fn(a) -&gt;\n    Generator(b)) -&gt; Generator(b)\n Transforms a generator into another one based on its generated values.\n\n The random value generated by the given generator is fed into the `do`\n function and the returned generator is used as the new generator.\n \n ## Examples\n \n `then` is a really powerful function, almost all functions exposed by this\n library could be defined in term of it!\n Take as an example `map`, it can be implemented like this:\n \n \n fn map(generator: Generator(a), with fun: fn(a) -&gt; b) -&gt; Generator(b) {\n   random.then(generator, fn(value) {\n     random.constant(fun(value))\n   })\n }\n \n \n Notice how the `do` function needs to return a `Generator(b)`, you can\n achieve that by wrapping any constant value with the `random.constant`\n generator.\n \n &gt; Code written with `then` can gain a lot in readability if you use the\n &gt; `use` syntax, especially if it has some deep nesting. As an example, this\n &gt; is how you can rewrite the previous example taking advantage of `use`:\n &gt;\n &gt; \n &gt; fn map(generator: Generator(a), with fun: fn(a) -&gt; b) -&gt; Generator(b) {\n &gt;   use value &lt;- random.then(generator)\n &gt;   random.constant(fun(value))\n &gt; }\n &gt; \n \nSynonyms:\nprng/random.then\nprng/random then","url":"prng/random.html#then"},{"doc":"prng/random","title":"to_iterator","content":"pub fn to_iterator(generator: Generator(a), seed: Seed) -&gt; Iterator(\n  a,\n)\n Turns the given generator into an infinite stream of random values generated\n with it.\n \n `seed` is the seed used to get the initial random value and start the\n infinite sequence.\n \n If you don&#39;t care about the initial seed and reproducibility is not your\n goal, you can use `sample` which works like this function and randomly picks\n the initial seed.\n \nSynonyms:\nprng/random.to_iterator\nprng/random to_iterator","url":"prng/random.html#to_iterator"},{"doc":"prng/random","title":"try_uniform","content":"pub fn try_uniform(options: List(a)) -&gt; Result(Generator(a), Nil)\n This function works exactly like `uniform` but will return an `Error(Nil)`\n if the provided argument is an empty list since the generator wouldn&#39;t be\n able to produce any value in that case.\n \n It generates values from the given list with equal probability.\n \n ## Examples\n \n \n random.try_uniform([])\n // -&gt; Error(Nil)\n \n \n For example if you consider the following type definition to model color:\n \n \n type Color {\n   Red\n   Green\n   Blue\n }\n \n \n This call of `try_uniform` will produce a generator wrapped in an `Ok`:\n \n \n let assert Ok(color_1) = random.try_uniform([Red, Green, Blue])\n let color_2 = random.uniform(Red, [Green, Blue])\n \n \n The generators `color_1` and `color_2` will behave exactly the same.\n \nSynonyms:\nprng/random.try_uniform\nprng/random try_uniform","url":"prng/random.html#try_uniform"},{"doc":"prng/random","title":"try_weighted","content":"pub fn try_weighted(options: List(#(Float, a))) -&gt; Result(\n  Generator(a),\n  Nil,\n)\n This function works exactly like `weighted` but will return an `Error(Nil)`\n if the provided argument is an empty list since the generator wouldn&#39;t be\n able to produce any value in that case.\n \n It generates values from the given list with a weighted probability.\n \n ## Examples\n \n \n random.try_weighted([])\n // -&gt; Error(Nil)\n \n \n For example if you consider the following type definition to model color:\n \n \n type CoinFlip {\n   Heads\n   Tails\n }\n \n \n This call of `try_weighted` will produce a generator wrapped in an `Ok`:\n \n \n let assert Ok(coin_1) =\n   random.try_weighted([#(0.75, Heads), #(0.25, Tails)])\n let coin_2 = random.uniform(#(0.75, Heads), [#(0.25, Tails)])\n \n \n The generators `coin_1` and `coin_2` will behave exactly the same.\n \nSynonyms:\nprng/random.try_weighted\nprng/random try_weighted","url":"prng/random.html#try_weighted"},{"doc":"prng/random","title":"uniform","content":"pub fn uniform(first: a, others: List(a)) -&gt; Generator(a)\n Generates values from the given ones with an equal probability.\n \n This generator can guarantee to produce values since it always takes at\n least one item (as its first argument); if it were to accept just a list of\n options, it could be called like this:\n \n \n uniform([])\n \n \n In which case it would be impossible to actually produce any value: none was\n provided!\n \n ## Examples\n \n Given the following type to model colors:\n \n \n pub type Color {\n   Red\n   Green\n   Blue\n }\n \n \n You could write a generator that returns each color with an equal\n probability (~33%) each color like this:\n \n \n let color = random.uniform(Red, [Green, Blue])\n \n \nSynonyms:\nprng/random.uniform\nprng/random uniform","url":"prng/random.html#uniform"},{"doc":"prng/random","title":"weighted","content":"pub fn weighted(first: #(Float, a), others: List(#(Float, a))) -&gt; Generator(\n  a,\n)\n Generates values from the given ones with a weighted probability.\n \n This generator can guarantee to produce values since it always takes at\n least one item (as its first argument); if it were to accept just a list of\n options, it could be called like this:\n \n \n uniform([])\n \n \n In which case it would be impossible to actually produce any value: none was\n provided!\n \n ## Examples\n \n Given the following type to model the outcome of a coin flip:\n \n \n pub type CoinFlip {\n   Heads\n   Tails\n }\n \n \n You could write a generator for a loaded coin that lands on head 75% of the\n times like this:\n \n \n let loaded_coin = random.weighted(#(Heads, 0.75), [#(Tails, 0.25)])\n \n \n In this example the weights add up to 1, but you could use any number: the\n weights get added up to a `total` and the probability of each option is its\n `weight` / `total`.\n \nSynonyms:\nprng/random.weighted\nprng/random weighted","url":"prng/random.html#weighted"},{"doc":"prng/random","title":"prng/random","content":" This package provides many building blocks that can be used to define\n pure generators of pseudo-random values.\n \n This is based on the great\n [Elm implementation](https://package.elm-lang.org/packages/elm/random/1.0.0/)\n of [Permuted Congruential Generators](https://www.pcg-random.org).\n \n _It is not cryptographically secure!_\n ","url":"prng/random.html"},{"doc":"prng","title":"main","content":"pub fn main() -&gt; List(Int)\n\nSynonyms:\nprng.main\nprng main","url":"prng.html#main"},{"doc":"prng","title":"prng","content":"","url":"prng.html"}]);